/// <reference types="bn.js" />
import { Connection, PublicKey } from "@solana/web3.js";
import { EscrowFetcher as FetcherOld } from "escrow-sdk-old";
import { EscrowFetcher as Fetcher, OfferDataInfo, OfferQuotes, MarketClient } from "../../index";
import { ITradeDataInfo, Version } from "../types";
import { BN } from "@project-serum/anchor";
import { Wallet } from "@project-serum/anchor/dist/cjs/provider";
import { TransactionBuilder } from "@orca-so/common-sdk";
import Decimal from "decimal.js";
export declare class EscrowRouter {
    pubkeyOld?: PublicKey;
    configIdOld?: string;
    pubkey: PublicKey;
    marketId: string;
    configId: string;
    fetcherOld?: FetcherOld;
    fetcher: Fetcher;
    constructor(connection: Connection, pubkey: PublicKey, marketId: string, configId: string, pubkeyOld?: PublicKey, configIdOld?: string);
    /**
     * @dev get trade by status and side optinally
     * @param tradeStatus : Awaiting | Deposited | Disputed | Released | Canceled
     * @param side  : Buy | Sell (if not provided, will return both buy and sell)
     * @returns ITradeDataInfo[]
     */
    getTradesByStatus(tradeStatus: object, side?: object): Promise<ITradeDataInfo[]>;
    getOffersByStatus(offerStatus: object, side?: object): Promise<OfferDataInfo[]>;
    getTradesByBuyerAddress(buyerAddress: PublicKey): Promise<(ITradeDataInfo | null)[]>;
    getTradesBySellerAddress(sellerAddress: PublicKey): Promise<(ITradeDataInfo | null)[]>;
    /**
     * a function to get trade data by trade id
     */
    getTradeDataByID(tradeId: BN, version?: Version, refresh?: boolean): Promise<ITradeDataInfo | null>;
    /**
     * a function to make a new offer
     * @param wallet user wallet
     * @param tokenMint mint of token that user want to create offer with
     * @param bondType type of bond. Can either be Spl or Native. Use "BondType" value from sdk
     * @param minTradeSize minimum size of trade can be created from the offer. Should be between "0.2" and "1"
     * @param tokenAmount total amount of token that user want to create offer with
     * @param fiatAmount equivalent number of fiat amount
     * @returns
     */
    makeOffer(wallet: Wallet, tokenMint: PublicKey, side: object, bondType: object, minTradeSize: Decimal, tokenAmount: BN, fiatAmount: BN, encryptedBankName: string | null, encryptedAccountNumber: string | null, encryptedAccountName: string | null, nonce: string | null): Promise<{
        tx: TransactionBuilder;
        id: BN;
    }>;
    matchingEngine(marketClient: MarketClient, user: PublicKey, tradeAmount: BN, offerQuotes: OfferQuotes, useBondless: boolean): Promise<{
        tx: TransactionBuilder;
        ids: BN[];
    }>;
    cancelTrade(wallet: Wallet, tradeData: ITradeDataInfo): Promise<TransactionBuilder>;
    cancelOffer(wallet: Wallet, offerId: BN): Promise<TransactionBuilder>;
    release(wallet: Wallet, tradeData: ITradeDataInfo): Promise<TransactionBuilder>;
    callArbitrator(wallet: Wallet, tradeData: ITradeDataInfo): Promise<TransactionBuilder>;
    resolveDispute(wallet: Wallet, tradeData: ITradeDataInfo, disputeAction: object, compensationAction: object): Promise<TransactionBuilder>;
}
